//--------------------------------------------------------------------- 
#include "MainWindow.h" 
//--------------------------------------------------------------------- 
BAppMainWindow::BAppMainWindow(BRect frame,const char *title) 
    :BWindow(frame,title,MAINWINDOW_WINDOWSTYLE,0) 
{ 
    mainview=new BAppMainView(Bounds());    
    AddChild(mainview); 
    save_filepanel=new BFilePanel(B_SAVE_PANEL); 
    editingpath=NULL; 

    mainview->MakeFocus(); 
} 
//--------------------------------------------------------------------- 
BAppMainWindow::~BAppMainWindow() 
{ 
    delete save_filepanel; 
    if(editingpath!=NULL) 
        delete editingpath; 
} 
//--------------------------------------------------------------------- 
void BAppMainWindow::MessageReceived(BMessage *msg) 
{ 
    switch(msg->what) 
    { 
        case B_UNDO: 
        case B_COPY: 
        case B_CUT: 
        case B_PASTE: 
        case B_SELECT_ALL: 
            ((BMemoView *)FindView("memo"))->MessageReceived(msg); 
            break; 
        case MSG_SAVE: 
            msg->FindBool("to_quit",&save_to_quit); 
                        
            if(editingpath!=NULL) 
            { 
                BEntry entry; 
                entry.SetTo(editingpath->Path()); 
                SaveFile(&entry); 
                break; 
            } 
        case MSG_SAVEAS: 
            msg->FindBool("to_quit",&save_to_quit); 

            save_filepanel->SetTarget(this); 
            save_filepanel->Show(); 
            break; 
        case B_SAVE_REQUESTED: 
            { 
                entry_ref ref; 
                BString nam; 
                BDirectory dir; 
                BEntry entry; 

                msg->FindRef("directory",&ref); 
                msg->FindString("name",&nam); 
                dir.SetTo(&ref); 

                if(entry.SetTo(&dir,nam.String())==B_OK) 
                { 
                    SaveFile(&entry); 
                    return; 
                } 
            } 
            break;        
        default: 
            BWindow::MessageReceived(msg); 
    } 
} 
//--------------------------------------------------------------------- 
BAppMainView::BAppMainView(BRect frame) 
    :BView(frame,"bappmainview",B_FOLLOW_ALL,B_WILL_DRAW) 
{ 
    BRect viewrect(Bounds()); 
    BMenuBar *mainmenu=new BMenuBar(BRect(0,0,viewrect.right, 
                                    be_plain_font->Size()*2), 
                                    "mainmenubar"); 
    AddChild(mainmenu); 

    memo=new BMemoView(BRect(0,be_plain_font->Size()*2, 
                             viewrect.right-B_V_SCROLL_BAR_WIDTH, 
                             viewrect.bottom-B_H_SCROLL_BAR_HEIGHT), 
                       "memo",B_FOLLOW_ALL); 
    BScrollView *scr=new BScrollView("memoscroll",memo, 
                                     B_FOLLOW_ALL,0,true,true); 
    AddChild(scr); 

    BMenu *filemenu=new BMenu("File"); 
    filemenu->AddItem(new BMenuItem("Open...",new BMessage(MSG_OPEN))); 
    filemenu->ItemAt(0)->SetTarget(be_app); 
    filemenu->AddItem(new BMenuItem("New Text",new BMessage(MSG_ADDEDITOR))); 
    filemenu->ItemAt(1)->SetTarget(be_app); 
    filemenu->AddItem(new BMenuItem("Save",new BMessage(MSG_SAVE))); 
    filemenu->ItemAt(2)->Message()->AddBool("to_quit",false); 
    filemenu->AddItem(new BMenuItem("Save As...",new BMessage(MSG_SAVEAS))); 
    filemenu->ItemAt(3)->Message()->AddBool("to_quit",false); 
    filemenu->AddSeparatorItem(); 
    filemenu->AddItem(new BMenuItem("Close",new BMessage(B_QUIT_REQUESTED))); 
    filemenu->AddSeparatorItem(); 
    filemenu->AddItem(new BMenuItem("Quit",new BMessage(MSG_QUIT))); 
    filemenu->ItemAt(7)->SetTarget(be_app); 
    mainmenu->AddItem(filemenu); 

    BMenu *editmenu=new BMenu("Edit"); 
    editmenu->AddItem(new BMenuItem("Undo",new BMessage(B_UNDO))); 
    editmenu->AddSeparatorItem(); 
    editmenu->AddItem(new BMenuItem("Copy",new BMessage(B_COPY))); 
    editmenu->AddItem(new BMenuItem("Cut",new BMessage(B_CUT))); 
    editmenu->AddItem(new BMenuItem("Paste",new BMessage(B_PASTE))); 
    editmenu->AddItem(new BMenuItem("Select All",new BMessage(B_SELECT_ALL))); 
    mainmenu->AddItem(editmenu); 

    memo->SetDoesUndo(true); 
    memo->MakeFocus(); 
} 
//--------------------------------------------------------------------- 
void BAppMainWindow::OpenFile(BEntry *entry) 
{ 
    if(entry->Exists()) 
    { 
        BFile fil; 
        off_t size; 

        try 
        { 
            fil.SetTo(entry,B_READ_ONLY | B_FAIL_IF_EXISTS); 
            fil.GetSize(&size); 
            ((BTextView *)FindView("memo"))->SetText(&fil,0,size); 
            fil.Unset(); 
             
            SetFileName(entry); 
        } 
        catch(...) 
        { 
            BAlert *altmsg = new BAlert("Error", 
                                        "File read failed!!","OK", 
                                        NULL,NULL,B_WIDTH_AS_USUAL, 
                                        B_WARNING_ALERT); 
            altmsg->Go(); 
        } 
    } 
} 
//--------------------------------------------------------------------- 
void BAppMainWindow::SetFileName(BEntry *entry) 
{ 
    if(editingpath==NULL) 
        editingpath=new BPath(); 
    entry->GetPath(editingpath); 
    
    if(editingpath!=NULL) 
    { 
        char nam[B_FILE_NAME_LENGTH]; 
        
        entry->GetName(nam); 
        SetTitle(nam); 
    } 

    ((BMemoView *)FindView("memo"))->Modified=false; 
} 
//--------------------------------------------------------------------- 
void BAppMainWindow::SaveFile(BEntry *entry) 
{ 
    BFile fil; 
    off_t wsize; 
    off_t tsize; 

    try 
    { 
        fil.SetTo(entry,B_READ_WRITE | B_CREATE_FILE); 
        tsize=strlen(((BTextView *)FindView("memo"))->Text()); 
        wsize=fil.WriteAt(0,((BTextView *)FindView("memo"))->Text(), 
                  tsize); 
        
        if(tsize==wsize) 
        { 
            SetFileName(entry); 
            
            if(save_to_quit) 
                PostMessage(new BMessage(B_QUIT_REQUESTED)); 
        } 
    } 
    catch(...) 
    { 
        BAlert *altmsg = new BAlert("Error", "File write failed!!","OK", 
                        NULL,NULL,B_WIDTH_AS_USUAL, 
                        B_WARNING_ALERT); 
        altmsg->Go(); 
    } 
} 
//--------------------------------------------------------------------- 
bool BAppMainWindow::QuitRequested() 
{ 
    BMemoView *memo; 
    bool result; 
    
    memo=((BMemoView *)FindView("memo")); 
     
    result=false; 
    if(memo!=NULL) 
    { 
        if(memo->Modified) 
        { 
            char m[1024]; 
            int32 idx; 
            BAlert *wrnmsg; 
            
            sprintf(m,"Save changes to the document \"%s\"?", 
                    (editingpath!=NULL)?editingpath->Leaf():"Untitled"); 

            wrnmsg=new BAlert("Warning!!", m, 
                      "Cancel","Don't save","Save", 
                              B_WIDTH_AS_USUAL,B_OFFSET_SPACING, 
                              B_WARNING_ALERT); 
            wrnmsg->SetShortcut(0,B_ESCAPE); 
            idx=wrnmsg->Go(); 
            if(idx==0) 
                return false; 
            if(idx==2) 
            { 
                BMessage *msg=new BMessage(MSG_SAVE); 
                msg->AddBool("to_quit",true); 

                PostMessage(msg);                   
                return false; 
            } 
        } 
    } 

    BMessage *msg_closed=new BMessage(MSG_CLOSED); 
    msg_closed->AddPointer("wnd",this); 
    be_app->PostMessage(msg_closed); 

    return true; 
}; 
//--------------------------------------------------------------------